/*
  MySQL SQL query script for Tournament database
  Designed for MariaDB RDBMS

  Author:
    Pekka Helenius <fincer89 [at] hotmail [dot] com>
    2019

  Create new tables into the database
  Table creation order: follow primary & foreign key relations
  Primary keys must exist before adding linked foreign keys
  
  NOTE:
   - Does not create indexing for tables
   - Does not define ENGINE
   - Requires MariaDB >= 10.2 (for CHECK constraints)
   - Tested with MariaDB 10.3.14
*/

-- -----------------------------------------------------
-- Create new database if needed
CREATE SCHEMA IF NOT EXISTS tournament_db;

-- -----------------------------------------------------
-- Use the new database
USE tournament_db;

-- -----------------------------------------------------
-- Create new table COUNTRY
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS COUNTRY (
  id   CHAR(2)      NOT NULL UNIQUE,
  name NVARCHAR(20) NOT NULL,
  PRIMARY KEY (id)
);

-- CREATE UNIQUE INDEX `COUNTRY.id.IDX_UNIQUE` ON COUNTRY (id ASC);

-- -----------------------------------------------------
-- Create new table CITY
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS CITY (
  postalcode  VARCHAR(10)   NOT NULL,
  country_id  CHAR(2)       NOT NULL,
  name        NVARCHAR(100) NOT NULL,
  PRIMARY KEY (postalcode, country_id),
  -- CONSTRAINT `FK1.CITY`
    FOREIGN KEY (country_id)
    REFERENCES  COUNTRY (id)
);

-- CREATE INDEX `FK1.CITY.IDX`                      ON CITY (country_id ASC);
-- CREATE UNIQUE INDEX `CITY.postalcode.IDX_UNIQUE` ON CITY (postalcode ASC);

-- -----------------------------------------------------
-- Create new table TEAM
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS TEAM (
  id           CHAR(4)       NOT NULL UNIQUE,
  name         NVARCHAR(50)  NOT NULL,
  email        NVARCHAR(200) NOT NULL,
  phone        VARCHAR(25)   NULL,
  postalcode   VARCHAR(10)   NULL,
  street       NVARCHAR(80)  NULL,
  housenumber  NVARCHAR(7)   NULL,
  IBAN         VARCHAR(30)   NULL,
  SWIFT_bank   VARCHAR(15)   NULL,
  SWIFT_number VARCHAR(20)   NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK1.TEAM`
    FOREIGN KEY (postalcode)
    REFERENCES  CITY (postalcode)
);

-- CREATE INDEX `FK1.TEAM.IDX`              ON TEAM (postalcode ASC);
-- CREATE UNIQUE INDEX `TEAM.id.IDX_UNIQUE` ON TEAM (id ASC);

-- -----------------------------------------------------
-- Create new table PLAYER
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS PLAYER (
  id          CHAR(4)       NOT NULL UNIQUE,
  team_id     CHAR(4)       NOT NULL,
  firstname   NVARCHAR(30)  NOT NULL,
  lastname    NVARCHAR(30)  NOT NULL,
  email       NVARCHAR(200) NULL,
  phone       VARCHAR(25)   NULL,
  postalcode  VARCHAR(10)   NULL,
  street      NVARCHAR(80)  NULL,
  housenumber NVARCHAR(7)   NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK2.PLAYER`
    FOREIGN KEY (postalcode)
    REFERENCES  CITY (postalcode),
  -- CONSTRAINT `FK1.PLAYER`
    FOREIGN KEY (team_id)
    REFERENCES  TEAM (id)
);

-- CREATE INDEX `FK1.PLAYER.IDX`              ON PLAYER (postalcode ASC);
-- CREATE INDEX `FK2.PLAYER.IDX`              ON PLAYER (team_id ASC);
-- CREATE UNIQUE INDEX `PLAYER.id.IDX_UNIQUE` ON PLAYER (id ASC);

-- -----------------------------------------------------
-- Create new table ORGANIZER_ORG
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS ORGANIZER_ORG (
  id          CHAR(4)       NOT NULL UNIQUE,
  name        NVARCHAR(50)  NOT NULL,
  email       NVARCHAR(200) NULL,
  phone       VARCHAR(25)   NULL,
  postalcode  VARCHAR(10)   NULL,
  street      NVARCHAR(80)  NULL,
  housenumber NVARCHAR(7)   NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK1.ORGANIZER_ORG`
    FOREIGN KEY (postalcode)
    REFERENCES  CITY (postalcode)
);

-- CREATE INDEX `FK1.ORGANIZER_ORG.IDX`              ON ORGANIZER_ORG (postalcode ASC);
-- CREATE UNIQUE INDEX `ORGANIZER_ORG.id.IDX_UNIQUE` ON ORGANIZER_ORG (id ASC);

-- -----------------------------------------------------
-- Create new table ORG_EMPLOYEE
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS ORG_EMPLOYEE (
  id                CHAR(4)       NOT NULL UNIQUE,
  organizer_org_id  CHAR(4)       NOT NULL,
  firstname         NVARCHAR(30)  NOT NULL,
  lastname          NVARCHAR(30)  NOT NULL,
  email             NVARCHAR(200) NULL,
  phone             VARCHAR(25)   NULL,
  postalcode        VARCHAR(10)   NULL,
  street            NVARCHAR(80)  NULL,
  housenumber       NVARCHAR(7)   NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK2.ORG_EMPLOYEE`
    FOREIGN KEY (postalcode)
    REFERENCES  CITY (postalcode),
  -- CONSTRAINT `FK1.ORG_EMPLOYEE`
    FOREIGN KEY (organizer_org_id)
    REFERENCES  ORGANIZER_ORG (id)
);

-- CREATE INDEX `FK1.ORG_EMPLOYEE.IDX`              ON ORG_EMPLOYEE (postalcode ASC);
-- CREATE INDEX `FK2.ORG_EMPLOYEE.IDX`              ON ORG_EMPLOYEE (organizer_org_id ASC);
-- CREATE UNIQUE INDEX `ORG_EMPLOYEE.id.IDX_UNIQUE` ON ORG_EMPLOYEE (id ASC);

-- -----------------------------------------------------
-- Create new table CLUBROOM
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS CLUBROOM (
  id          CHAR(4)      NOT NULL UNIQUE,
  name        NVARCHAR(50) NOT NULL,
  postalcode  VARCHAR(10)  NULL,
  street      NVARCHAR(80) NULL,
  housenumber NVARCHAR(7)  NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK1.CLUBROOM`
    FOREIGN KEY (postalcode)
    REFERENCES  CITY (postalcode)
);

-- CREATE INDEX `FK1.CLUBROOM.IDX`              ON CLUBROOM (postalcode ASC);
-- CREATE UNIQUE INDEX `CLUBROOM.id.IDX_UNIQUE` ON CLUBROOM (id ASC);

-- -----------------------------------------------------
-- Create new table GAME_EVENT
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS GAME_EVENT (
  id                    CHAR(4)       NOT NULL UNIQUE,
  clubroom_id           CHAR(4)       NULL,
  name                  NVARCHAR(50)  NOT NULL,
  starttime             DATETIME      NULL,
  endtime               DATETIME      NULL,
  event_fee             INT           NULL,
  extrainfo             NVARCHAR(500) NULL,
  PRIMARY KEY (id),
  -- CONSTRAINT `FK1.GAME_EVENT`
    FOREIGN KEY (clubroom_id)
    REFERENCES  CLUBROOM (id)
);

-- CREATE INDEX `FK1.EVENT.IDX`              ON GAME_EVENT (clubroom_id ASC);
-- CREATE UNIQUE INDEX `EVENT.id.IDX_UNIQUE` ON GAME_EVENT (id ASC);

-- -----------------------------------------------------
-- Create new joining table RESERVATION
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS RESERVATION (
  organizer_org_id  CHAR(4)  NOT NULL,
  clubroom_id       CHAR(4)  NOT NULL,
  time         DATETIME NULL,
  PRIMARY KEY (organizer_org_id, clubroom_id),
  -- CONSTRAINT `FK2.RESERVATION`
    FOREIGN KEY (clubroom_id)
    REFERENCES  CLUBROOM (id),
  -- CONSTRAINT `FK1.RESERVATION`
    FOREIGN KEY (organizer_org_id)
    REFERENCES  ORGANIZER_ORG (id)
);

-- CREATE INDEX `FK1.RESERVATION.IDX` ON RESERVATION (clubroom_id ASC);

-- -----------------------------------------------------
-- Create new joining table ARRANGEMENT
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS ARRANGEMENT (
  organizer_org_id CHAR(4) NOT NULL,
  event_id         CHAR(4) NOT NULL,
  PRIMARY KEY (organizer_org_id, event_id),
  -- CONSTRAINT `FK1.ARRANGEMENT`
    FOREIGN KEY (organizer_org_id)
    REFERENCES  ORGANIZER_ORG (id),
  -- CONSTRAINT `FK2.ARRANGEMENT`
    FOREIGN KEY (event_id)
    REFERENCES  GAME_EVENT (id)
);

-- CREATE INDEX `FK2.ARRANGEMENT.IDX` ON ARRANGEMENT (event_id ASC);

-- -----------------------------------------------------
-- Create new table TEAM_STATUS
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS TEAM_STATUS (
  team_id     CHAR(4)     NOT NULL,
  event_id    CHAR(4)     NOT NULL,
  team_status VARCHAR(20) NOT NULL,
  rank        INT         NULL,
  PRIMARY KEY (team_id, event_id),
  -- CONSTRAINT `FK2.TEAM_STATUS`
    FOREIGN KEY (event_id)
    REFERENCES  GAME_EVENT (id),
  -- CONSTRAINT `FK1.TEAM_STATUS`
    FOREIGN KEY (team_id)
    REFERENCES  TEAM (id),

-- MariaDB 10.2 implements CHECK constraints
-- https://mariadb.com/kb/en/library/changes-improvements-in-mariadb-102/
-- Determine your version with query: SELECT @@version

  CHECK (rank BETWEEN 1 AND 999),
  CHECK (
    team_status = 'playing'    OR
    team_status = 'registered' OR
    team_status = 'played'     OR
    team_status = 'ranked'
  )

);

-- CREATE INDEX `FK1.TEAM_STATUS.IDX` ON TEAM_STATUS (event_id ASC);

-- -----------------------------------------------------
-- Create new (joining) table PLAYER_RANK
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS PLAYER_RANK (
  player_id CHAR(4) NOT NULL UNIQUE,
  team_id   CHAR(4) NOT NULL,
  rank      INT     NOT NULL,
  PRIMARY KEY (player_id, team_id),
  -- CONSTRAINT `FK1.PLAYER_RANK`
    FOREIGN KEY (player_id)
    REFERENCES  PLAYER (id),
  -- CONSTRAINT `FK2.PLAYER_RANK`
    FOREIGN KEY (team_id)
    REFERENCES  TEAM (id),

-- MariaDB 10.2 implements CHECK constraints
-- https://mariadb.com/kb/en/library/changes-improvements-in-mariadb-102/
-- Determine your version with query: SELECT @@version

  CHECK (rank BETWEEN 1 AND 99)
);

-- CREATE INDEX `FK1.PLAYER_RANK.IDX`                     ON PLAYER_RANK (player_id ASC);
-- CREATE INDEX `FK2.PLAYER_RANK.IDX`                     ON PLAYER_RANK (team_id ASC);
-- CREATE UNIQUE INDEX `PLAYER_RANK.player_id.IDX_UNIQUE` ON PLAYER_RANK (player_id ASC);

-- -----------------------------------------------------
-- Create new table TEAM_RANK_GLOBAL
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS TEAM_RANK_GLOBAL (
  rank    INT     NOT NULL UNIQUE,
  team_id CHAR(4) NOT NULL UNIQUE,
  PRIMARY KEY (rank, team_id),
  -- CONSTRAINT `FK1.TEAM_RANK`
    FOREIGN KEY (team_id)
    REFERENCES  TEAM (id),

-- MariaDB 10.2 implements CHECK constraints
-- https://mariadb.com/kb/en/library/changes-improvements-in-mariadb-102/
-- Determine your version with query: SELECT @@version

  CHECK (rank BETWEEN 1 AND 9999)
);

-- CREATE INDEX `FK1.TEAM_RANK.IDX`                   ON TEAM_RANK_GLOBAL (team_id ASC);
-- CREATE UNIQUE INDEX `TEAM_RANK.rank_UNIQUE`        ON TEAM_RANK_GLOBAL (rank ASC);
-- CREATE UNIQUE INDEX `TEAM_RANK.team_id.IDX_UNIQUE` ON TEAM_RANK_GLOBAL (team_id ASC);

-- -----------------------------------------------------
-- Create new table BETTOR
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS BETTOR (
  id           NVARCHAR(20)  NOT NULL,
  firstname    NVARCHAR(30)  NOT NULL,
  lastname     NVARCHAR(30)  NOT NULL,
  email        NVARCHAR(200) NOT NULL,
  phone        VARCHAR(25)   NULL,
  IBAN         VARCHAR(30)   NULL,
  SWIFT_bank   VARCHAR(15)   NULL,
  SWIFT_number VARCHAR(20)   NULL,
  password     CHAR(128)     NOT NULL,
  PRIMARY KEY (id)
);

-- -----------------------------------------------------
-- Create new table BETTING
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS BETTING (
  bettor_id         NVARCHAR(20) NOT NULL,
  event_id          CHAR(4)      NOT NULL,
  team_id           CHAR(4)      NOT NULL,
  team_rank_guess   INT          NOT NULL,
  multiplier        DECIMAL(4,2) NOT NULL,
  initmoney         DECIMAL(5,2) NOT NULL,
  betstatus         NVARCHAR(20) NOT NULL,
  PRIMARY KEY (bettor_id, event_id, team_id),
  -- CONSTRAINT `FK1.BETTING`
    FOREIGN KEY (bettor_id)
    REFERENCES  BETTOR (id),
  -- CONSTRAINT `FK3.BETTING`
    FOREIGN KEY (team_id)
    REFERENCES  TEAM (id),
  -- CONSTRAINT `FK2.BETTING`
    FOREIGN KEY (event_id)
    REFERENCES  GAME_EVENT (id),

-- MariaDB 10.2 implements CHECK constraints
-- https://mariadb.com/kb/en/library/changes-improvements-in-mariadb-102/
-- Determine your version with query: SELECT @@version

  CHECK (team_rank_guess BETWEEN 1 AND 99),
  CHECK (multiplier      BETWEEN 1 AND 10),
  CHECK (initmoney       BETWEEN 1 AND 999),
  CHECK (
    betstatus = 'queued for approval' OR
    betstatus = 'accepted' OR
    betstatus = 'rejected' OR
    betstatus = 'cancelled' OR
    betstatus = 'no win' OR
    betstatus = 'won' OR
    betstatus = 'other'
  )
);

-- CREATE INDEX `FK3.BETTING.IDX` ON BETTING (team_id ASC);
-- CREATE INDEX `FK2.BETTING.IDX` ON BETTING (event_id ASC);
