/*
  MySQL SQL query script for Tournament database
  Designed for MariaDB RDBMS

  Author:
    Pekka Helenius <fincer89 [at] hotmail [dot] com>
    2019
*/

-- -----------------------------------------------------
-- Assumes that database 'tournament_db' exists
USE tournament_db;

-- -----------------------------------------------------
/*
  ASSIGNMENT 1

Database company owning the tournament database has changed its terms of use for the tournament application platform. Major changes for players include requirement of having personal contact information connected to new and existing accounts. To be specific, the required information is:

- phone number
- postal code
- street
- house number

Earlier this information was optional and only email information was required.

A database administrator sends a group email to notify players about the changed terms of use. The administrator wants to send the email only for players lacking the required contact information to avoid spamming messages unnecessarily. The email message has a clearly stated deadline after which the database company reserves rights to delete player information from the tournament database unless the new terms of use conditions have been met.

To avoid errors, lack of information and misunderstandings, the administrator wants to send the message to team common email addresses, as well. This is useful in cases where players can't be reached via their personal email addresses.

Create a SQL query which meets the administrator requirements.
  
*/
-- -----------------------------------------------------

SELECT
  P.firstname,
  P.lastname,
  P.email AS player_email,
  T.email AS team_email
FROM PLAYER P

  JOIN TEAM T
    ON P.team_id = T.id

  WHERE P.phone   IS NULL OR
    P.postalcode  IS NULL OR
    P.street      IS NULL OR
    P.housenumber IS NULL
;

-- -----------------------------------------------------
/*
  ASSIGNMENT 2

Find cities and towns which are not used or are not connected to the following data:
 - organization worker
 - organization
 - players
 - teams
 - clubrooms
 
Query results should contain a list of unused cities and towns, including possible country codes.

  Required query operations:

    1) Get all possible postal codes in the database and combine them into a single column

    2) Compare the combined column with the values of postalcode column in CITY table

    3) Print those values of postalcode column in CITY table which are not present in
       the combined query table generated in step 1)
  
*/
-- -----------------------------------------------------

SELECT CITY.* FROM (

    SELECT DISTINCT 
     postalcode AS postalcode_used
    FROM ORG_EMPLOYEE

    UNION

    SELECT DISTINCT 
      postalcode AS postalcode_used
    FROM ORGANIZER_ORG

    UNION

    SELECT DISTINCT 
      postalcode AS postalcode_used
    FROM PLAYER

    UNION

    SELECT DISTINCT 
      postalcode AS postalcode_used
    FROM TEAM

    UNION

    SELECT DISTINCT 
      postalcode AS postalcode_used
    FROM CLUBROOM

) USED_ZIPCODES

  RIGHT JOIN CITY ON
    USED_ZIPCODES.postalcode_used = CITY.postalcode

  WHERE USED_ZIPCODES.postalcode_used IS NULL
;

-- -----------------------------------------------------
/*
  ASSIGNMENT 3

Same description than in task 2. Instead of cities and towns, retrieve information of unused countries (country codes and county names)

*/
-- -----------------------------------------------------

SELECT COUNTRY.* FROM COUNTRY
  LEFT JOIN CITY ON
    COUNTRY.id = CITY.country_id

 WHERE CITY.name IS NULL
;

-- -----------------------------------------------------
/*
  ASSIGNMENT 4

Executives and strategy makers of the database company are interested in possibilities of the new tournament database. Therefore, they want some statistics based on the current data. Tournament database administrators receive a request and are tasked to fetch information regarding to tournament host countries and countries of participating teams. The executives want more information to estimate whether the company should increase number of available arenas and clubrooms for tournament events. The statistics will be used in an internal report which is appointed to the executives and the strategy makers.

The database administrators proceed and fetch the required information from the tournament database. To be specific, they print the following information:
  - tournament event name
  - tournament event host country
  - participated teams and their names
  - home country of each team

Printed output should have information whether a team has already played or just registered to an event. Additionally, the output should state whether an event has already been organized or not.

Sort the output primarily by tournament event names in descending order, and secondarily by team country names in descending order.

Output should also include tournament events without physical location (online events).

*/
-- -----------------------------------------------------

SELECT
  T.name AS event_name,
  (CASE WHEN MK.id IS NULL THEN 'online' ELSE MK.id END) AS event_country,
  (CASE WHEN CURDATE() > T.endtime THEN 'organized' ELSE 'planned' END) AS event_status,
  J.name AS team_name,
  MJ.id  AS team_country,
  JS.team_status
FROM TEAM J

  JOIN TEAM_STATUS JS
    ON JS.team_id = J.id

  JOIN GAME_EVENT T
    ON T.id = JS.event_id
    
  LEFT JOIN CLUBROOM K
    ON K.id = T.clubroom_id
    
  /* team postal code */
  JOIN CITY PKJ
    ON PKJ.postalcode = J.postalcode
  JOIN COUNTRY MJ
    ON MJ.id = PKJ.country_id

  /* clubroom postal code */
  LEFT JOIN CITY PKK
    ON PKK.postalcode = K.postalcode
  LEFT JOIN COUNTRY MK
    ON MK.id = PKK.country_id
    
  ORDER BY event_name ASC, team_country ASC
;

-- -----------------------------------------------------
/*
  TEHTÄVÄ 5

Which team has been the most popular guess among bettors? How many times the team has been betted on to? Fetch the information from the tournament database.

*/
-- -----------------------------------------------------

USE tournament_db;

SELECT
  J.name AS team_name,
  COUNT(V.team_id) AS team_guesses_total
FROM BETTING V

JOIN TEAM J
  ON J.id = V.team_id

  GROUP BY J.name, V.team_id
  ORDER BY team_guesses_total DESC
LIMIT 1;

-- -----------------------------------------------------
/*
  ASSIGNMENT 6

Print average ranking of each team in tournament events where it has been played in.

The print output should include

  - team name
  - how many times a team has been betted on to
  - average rank guess for a team in all tournament events among bettors
  - average rank of a team in all tournaments
  - number of tournaments a team has been played in

*/
-- -----------------------------------------------------

SELECT
  J.name AS team_name,
  COUNT(V.team_id) AS team_guesses_total,
  ROUND(AVG(V.team_rank_guess),2) AS team_rank_guess_AVG,
  (
    CASE
      WHEN
        AVG(JS.rank) IS NOT NULL
      THEN
        ROUND(AVG(JS.rank),2)
      ELSE
        'never ranked'
      END
  ) AS 'team_avg.rank',
  (
    CASE
      WHEN
        JS.team_status = 'played' OR JS.team_status = 'ranked'
      THEN
        COUNT(JS.team_id)
      ELSE
        0
      END
  ) AS team_gamestotal
FROM BETTING V

  JOIN TEAM J
    ON J.id = V.team_id

  JOIN TEAM_STATUS JS
   ON JS.team_id = V.team_id

  GROUP BY J.name, V.team_id, JS.team_status
  ORDER BY team_guesses_total DESC;

-- -----------------------------------------------------
/*
  ASSIGNMENT 7

The chief executive officer (CEO) of the database company want to know profits gained by the company and organizers from tournament events. Accountant suggest to get expenses, as well. The CEO agrees.

The accountant delivers the request to database administrators who promise to fetch the information from the tournament database.

*/
-- -----------------------------------------------------

SELECT
  PT.name AS event_name,
  SUM(PT.event_fee) AS event_profit,
  COUNT(JS.team_id) AS participated_teams_count
FROM TEAM_STATUS JS

JOIN GAME_EVENT PT
  ON JS.event_id = PT.id

GROUP BY PT.name;
  
-- -----------------------------------------------------
/*
  ASSIGNMENT 8

The database company board holds a meeting where an employee suggests to obtain better statistics about geographical diversity of tournament teams, players and bettors, not forgetting information about the most active userbase by countries around the world. Most participants agree with the employee.

Another employee delivers the request to tournament database administrators who promise to fetch the information for the company board.

Tournament user groups are as follows:
  - tournament event organizer employees
  - tournament event organizer
  - team players
  - team
  
Obtain a country-specific group statistics for the described user groups.
*/
-- -----------------------------------------------------

SELECT
  JT.id AS id,
  CONCAT('employee') AS 'group',
  M1.name AS country_name
FROM ORG_EMPLOYEE JT

  JOIN CITY PJT
    ON PJT.postalcode = JT.postalcode
  JOIN COUNTRY M1
    ON M1.id = PJT.country_id

UNION

SELECT
  JO.id AS id,
  CONCAT('organizer') AS 'group',
  M2.name AS country_name
FROM ORGANIZER_ORG JO

  JOIN CITY PJO
    ON PJO.postalcode = JO.postalcode
  JOIN COUNTRY M2
    ON M2.id = PJO.country_id

UNION

SELECT
  PE.id AS id,
  CONCAT('player') AS 'group',
  M3.name AS country_name
FROM PLAYER PE

  JOIN CITY PPE
    ON PPE.postalcode = PE.postalcode
  JOIN COUNTRY M3
    ON M3.id = PPE.country_id

UNION

SELECT
  JK.id AS id,
  CONCAT('team') AS 'group',
  M4.name AS country_name
FROM TEAM JK

  JOIN CITY PJK
    ON PJK.postalcode = JK.postalcode
  LEFT JOIN COUNTRY M4
    ON M4.id = PJK.country_id

ORDER BY country_name ASC;
