/*
  MySQL SQL query script for Tournament database
  Designed for MariaDB RDBMS

  Author:
    Pekka Helenius <fincer89 [at] hotmail [dot] com>
    2019
*/

-- -----------------------------------------------------
-- Assumes that database 'tournament_db' exists
USE tournament_db;

-- -----------------------------------------------------
/*
  ASSIGNMENT 9

Database team has recruited a new inexperienced employee. The new employee is quite unsure about structure of the tournament database, and he/she is missing common knowledge about primary and foreign keys. 

Being unaware of his/her inexperience, the database team gives him/her a task to add a new clubroom with the following information:

  - id: LAPP
  - name: L'Apparemment Café
  - postal code, city, maa: 75003, Quartier des Archives, France
  - street: Rue des Coutures Saint-Gervais 18

The operation fails. Why?

Verbal answer:

  - Clubroom postal code (reference key value) does not exist
    in the primary key CITY.postalcode
  - France does not exist/is not determined in the database at all

Therefore, keeping constraints and basic ruleset in mind, the whole operation should be
started by adding missing country data for France (COUNTRY table) after which adding
necessary postal code data is required (CITY table). Afterwards, adding the new
clubroom is possible.
  
*/
-- ---------------------------
-- FAILING/INVALID QUERY
-- INSERT INTO CLUBROOM (id, name, postalcode, street, housenumber) VALUES
--   ('LAPP', 'L\'Apparemment Café', '75003', 'Rue des Coutures Saint-Gervais', '18');

-- ---------------------------

-- SUCCEEDING/VALID QUERY

-- NOTE! Brancing condition: in later exercises, COUNTRY table contains new column 'id_a3'
-- At this moment, the column does not exist.
-- In this answer, both options are considered.

-- -----------------------------------------------------
/*
Because the user could have executed later presented ALTER queries even now,
we must determine column count of COUNTRY table before inserting any data.

Therefore, a new function and SQL procedure were developed.

FUNCTION: 
If 'XX' column exists, return value 1. Otherwise, return NULL.
Because ALTER queries affect column structure of a database, this function
is necessary. At this moment, we don't know whether the target table
contains 'XX' column or not.

PROCEDURE:
Proceed based on function return value. In other words, perform
different operation based on whether the 'XX' column exists or not.
Insert correct amount of data to the target table.
The procedure is quite unscalable but works in this case.

In this case, the referred target table is COUNTRY and the checked column is 'id_a3'

*/

DROP FUNCTION IF EXISTS columnCheck;
DROP PROCEDURE IF EXISTS INSERT_countryValues;

DELIMITER //

-- -------------
CREATE FUNCTION columnCheck(db VARCHAR(50), tbl VARCHAR(50), col VARCHAR(50))
  RETURNS INT
  DETERMINISTIC
  BEGIN
    RETURN IF((SELECT COUNT(*) AS COLUMN_RETURN FROM INFORMATION_SCHEMA.COLUMNS WHERE
              TABLE_SCHEMA = db AND TABLE_NAME = tbl AND COLUMN_NAME = col),
              1, NULL);
  END//

-- -------------
CREATE PROCEDURE INSERT_countryValues(
  database_input     VARCHAR(50),
  table_input        VARCHAR(50),
  column_input       VARCHAR(50),
  id_input           CHAR(2),
  id_a3_input        CHAR(3),
  dialing_code_input VARCHAR(5), 
  name_input         NVARCHAR(20)
)

BEGIN

  IF (SELECT columnCheck(database_input, table_input, column_input)) IS NOT NULL
  THEN
    INSERT INTO COUNTRY (id, id_a3, dialing_code, name)
      VALUES (id_input, id_a3_input, dialing_code_input, name_input)
        ON DUPLICATE KEY UPDATE
          id_a3        = id_a3_input,
          dialing_code = dialing_code_input,
          name         = name_input
    ;
  ELSE
    INSERT INTO COUNTRY (id, name)
      VALUES (id_input, name_input)
        ON DUPLICATE KEY UPDATE
          name = name_input
    ;
  END IF;

END//

DELIMITER ;
-- -----------------------------------------------------

-- 1) Add missing country information into COUNTRY table in tournament_db
CALL INSERT_countryValues('tournament_db', 'COUNTRY', 'id_a3', 'FR', 'FRA', '33', 'France');

-- 2) Add city information
INSERT INTO CITY (postalcode, country_id, name) VALUES
  ('75003', 'FR', 'Quartier des Archives');

-- 3) Add new clubroom
INSERT INTO CLUBROOM (id, name, postalcode, street, housenumber) VALUES
  ('LAPP', 'L\'Apparemment Café', '75003', 'Rue des Coutures Saint-Gervais', '18');

-- -----------------------------------------------------
/*
  ASSIGNMENT 10

Game tournament organizer LFG Australia gets information about new game tournament "2020 NSW Family Tournaments". The organizer wants to join our tournament platform and add information about the new game tournament immediately. The tournament has additional organizer, american Global Board Game Association, which determinates participation fee of 160.09 australian dollars (use equivalent euro value 99).

The tournament event has marketing description which states: "Unforgettable and very unique board game event in the middle of Australian summer, hard to forget! Join today."

The organizer has retrieved the following additional information about the event site: 

- Name:                The Games Cube
- Contact information: 79 Phillip Street, Parramatta, NSW 2150, Australia

Organizer information is as follows:

- Name:                LFG Australia
- Contact information: 12 Morehead Street, Redfern (Sydney), New South Wales, 2016, Australia
- Email:               firstname.lastname@@lfg-aus.com.au
- Phone:               unknown

Organizer contact person information is as follows:

- Name:                Sophia Timms
- Contact information: 11 R.L. Gambling Street, The Berri Barmera Council, 5343, Australia
- Phone:               +61(03)53840554
- Email:               sophia.timms@lfg-aus.com.au

Event calendar period is 16th - 17th May 2020.

Using this information, add a new event reservation with relevant data to the tournament database.
  
*/
-- -----------------------------------------------------

-- 1) Add new country (Australia)
CALL INSERT_countryValues('tournament_db', 'COUNTRY', 'id_a3', 'AU', 'AUS', '61', 'Australia');

-- 2) Add postal codes of the organizer, the employee and the tournament site (city)
INSERT INTO CITY (postalcode, country_id, name) VALUES
  ('2150', 'AU', 'Parramatta'),
  ('2016', 'AU', 'Redfern'),
  ('5343', 'AU', 'The Berri Barmera Council')
;

-- 3) Add the new organizer
INSERT INTO ORGANIZER_ORG (id, name, email, phone, postalcode, street, housenumber) VALUES
  ('O012', 'LFG Australia', 'firstname.lastname@@lfg-aus.com.au', NULL, '2016', 'Morehead Street', '12')
;

-- 4) Add the new employee
INSERT INTO ORG_EMPLOYEE (id, organizer_org_id, firstname, lastname, email, phone, postalcode, street, housenumber) VALUES
  ('T020', 'O012', 'Sophia', 'Timms', 'sophia.timms@lfg-aus.com.au', '+61(03)53840554', '5343', 'R.L. Gambling Street', '11')
;

-- 5) Add the new tournament event site (clubroom)
INSERT INTO CLUBROOM (id, name, postalcode, street, housenumber) VALUES
  ('GACU', 'The Games Cube', '2150', 'Phillip Street', '79')
;

-- 6) Add a new reservation
INSERT INTO RESERVATION (organizer_org_id, clubroom_id, time) VALUES
  ('O012', 'GACU', '2019-12-07 17:39:43.643')
;

-- 7) Add the new tournament event, link it to the new clubroom
INSERT INTO GAME_EVENT (id, clubroom_id, name, starttime, endtime, event_fee, extrainfo) VALUES
  ('E005', 'GACU', '2020 NSW Family Tournaments', '2020-05-16 00:00:00.000', '2020-05-17 23:59:59.999', 99, 'Unforgettable and very unique board game event in the middle of Australian summer, hard to forget! Join today.')
;

-- 8) Add a new arrangement
INSERT INTO ARRANGEMENT (organizer_org_id, event_id) VALUES
  ('O012', 'E005'),
  ('O002', 'E005')
;

-- -----------------------------------------------------
/*
  ASSIGNMENT 11

The database company receives registration information from a new team, known as Mahokara Team, from New Zealand. The team provides the following information:
  - Email:                mahorakateam@gmail.com
  - Contact information:  16 Majoribanks Street, Wellington, 6011, New Zealand
  - SWIFT bank:           ANZBNZ22058
  - SWIFT account number: 05835020297713
  - Global ranking:       156

At the moment, the team has three active players with the following information:

  Player 1
    - Name:                Alyssa Trickett
    - Contact information: 50 Edinburgh Drive, Boydtown (Queenstown), 9348, New Zealand
    - Phone:               +64(027)5721709
    - Email:               mohokai96@outlook.com

  Player 2
    - Name:                Jack Powers
    - Contact information: 9 Judge and Jury Drive, Lower Shotover (Queenstown), 9304, New Zealand
    - Phone:               unknown
    - Email:               jackthesparrow@gmail.com

  Player 3
    - Name:                Elizabeth Maltby
    - Contact information: 237 Fernhill Road, Fernhill (Queenstown), 7050, New Zealand
    - Phone:               +64(029)5280754
    - Email:               zibbaree123@gmail.com

Add the new team and its players into the tournament database.
  
*/

-- -----------------------------------------------------

-- 1) Add new country (team and its players)
CALL INSERT_countryValues('tournament_db', 'COUNTRY', 'id_a3', 'NZ', 'NZL', '64', 'New Zealand');

-- 1) Add new postal codes (team and its players)
INSERT INTO CITY (postalcode, country_id, name) VALUES
  ('6011', 'NZ', 'Wellington'),
  ('9348', 'NZ', 'Boydtown'),
  ('9304', 'NZ', 'Lower Shotover'),
  ('7050', 'NZ', 'Fernhill')
;

-- 3) Add the new team
INSERT INTO TEAM (id, name, email, phone, postalcode, street, housenumber, IBAN, SWIFT_bank, SWIFT_number) VALUES
  ('J010', 'Mahokara Team', 'mahorakateam@gmail.com', NULL, '6011', 'Majoribanks Street', '16', NULL, 'ANZBNZ22058', '05835020297713')
;

-- 4) Add the new players, link to the new team
INSERT INTO PLAYER (id, team_id, firstname, lastname, email, phone, postalcode, street, housenumber) VALUES
  ('P024', 'J010', 'Alyssa', 'Trickett', 'mohokai96@outlook.com', '+64(027)5721709', '9348', 'Edinburgh Drive', '50'),
  ('P025', 'J010', 'Jack', 'Powers', 'jackthesparrow@gmail.com', NULL, '9304', 'Judge and Jury Drive', '9'),
  ('P026', 'J010', 'Elizabeth', 'Maltby', 'zibbaree123@gmail.com', '+64(029)5280754', '7050', 'Fernhill Road', '237')
;

-- 5) Add team global ranking: new row to TEAM_RANK_GLOBAL table
-- In real case, this information is fetched from external source
-- and added programmatically to the tournament database
INSERT INTO TEAM_RANK_GLOBAL (rank, team_id) VALUES
  (156, 'J010')
;

-- 6) Not adding: player ranks in the team
--    Reason: Only participated players have this information in
--            the tournament database. The added team has not played yet.

-- -----------------------------------------------------
/*
  ASSIGNMENT 12
  
Organizer LFG Australia hires a new employee with the following information:

- Name:                Amy Rechner
- Contact information: 44 Highmount Drive, Hampton Park (Melbourne), 3976, Australia
- Phone:               +61(08)82654779
- Email:               amy.rechner@lfg-aus.com.au

Add the new employee for the organizer in the tournament database.

*/

-- -----------------------------------------------------

-- 1) Add new postal code the the employee
INSERT INTO CITY (postalcode, country_id, name) VALUES
  ('3976', 'AU', 'Hampton Park')
;

-- 2) Add new employee information and link the employee to the organizer
INSERT INTO ORG_EMPLOYEE (id, organizer_org_id, firstname, lastname, email, phone, postalcode, street, housenumber) VALUES
  ('T021', 'O012', 'Amy', 'Rechner', 'amy.rechner@lfg-aus.com.au', '+61(08)82654779', '3976', 'Highmount Drive', '44')
;

-- -----------------------------------------------------
/*
  ASSIGNMENT 13

Many teams have already retrieved a hint about the new tournament event "2020 NSW Family Tournaments" (added in task 10 above). The participating teams, which already have up-to-date information about themselves in the tournament database, are:

  - Mahokara Team
  - FreeRiders
  - Red Hats

Not only the teams have heard about the coming up event: two hardcore fans have also heard rumors about it from multiple players. As passionate gamblers, they want to bet on their favorite teams on the event. However, they have not registered on the tournament platform. Some of the players tell these fans about the betting opportunity after which they hurry up signing up to the tournament platform.

Bettor 1 bets only on his/her favorite team. Bettor 2 bets not only on his/her favorite team but also on the the same team than bettor 1 did.

Bettor information is as follows:

  Bettor 1
    - User name:            Ratishda
    - Name:                 Zossima Gorbunov
    - Email:                zosgorb@gmail.com
    - Phone:                +7(8442)443225
    - Password:             private4Scape (64b4455db5e4fd242a22e8ea5e57b15850be4cfc3a7e58f432013723d7053677a472235c07b9cc7b08d504dfc607758bac3a073e103843144bc880166ae8662d)
    - SWIFT bank:           JSBSRU2PXXX
    - SWIFT account number: 027807458042622
    
  Betting information:
    
    - Favorite team:             Red Hats
    - Guessed team rank:         1
    - Betting multiplier:        3.22
    - Initial money for betting: 425 (euro)
    - Betting status:            accepted

  Bettor 2
    - User name:            Tiakakuna
    - Name:                 Harrison Brier
    - Email:                harrkuna@gmail.com
    - Phone:                unknown
    - Password:             karakuna (4eef0fa919697f77cbd13179a90c9e0d90e15f127362c6d70161689e49fb79a1ac12b33efd128493870daa3b6918c68d0838501f73fbfbd1bccfc7faa01c5d29)
    - SWIFT bank:           BKIDNZ2A
    - SWIFT account number: 26387479571396

    Betting information 1:
    
    - Team:                      Mahokara Team
    - Guessed team rank:         1
    - Betting multiplier:        7.88
    - Initial money for betting: 50 (euro)
    - Betting status:            queued for approval
    
    Betting information 2:
    
    - Team:                      FreeRiders
    - Guessed team rank:         3
    - Betting multiplier:        9.5
    - Initial money for betting: 230 (euro)
    - Betting status:            queued for approval

Link the mentioned teams to the tournament event. Add new bettors and their information, and add their betting information using the above input.
  
*/
-- -----------------------------------------------------


-- 1) Add the teams to the TEAM_STATUS table. Since the event has not yet occured, we mark the teams as registered ones.
INSERT INTO TEAM_STATUS (team_id, event_id, team_status, rank) VALUES
  ('J010', 'E005', 'registered', NULL),
  ('J003', 'E005', 'registered', NULL),
  ('J004', 'E005', 'registered', NULL)
;

-- 2) Add the new bettors
INSERT INTO BETTOR (id, firstname, lastname, email, phone, IBAN, SWIFT_bank, SWIFT_number, password) VALUES

  ('Ratishda', 'Zossima', 'Gorbunov', 'zosgorb@gmail.com', '+7(8442)443225', NULL, 'JSBSRU2PXXX', '027807458042622',
    '64b4455db5e4fd242a22e8ea5e57b15850be4cfc3a7e58f432013723d7053677a472235c07b9cc7b08d504dfc607758bac3a073e103843144bc880166ae8662d'),

  ('Tiakakuna', 'Harrison', 'Brier', 'harrkuna@gmail.com', NULL, NULL, 'BKIDNZ2A', '26387479571396',
    '4eef0fa919697f77cbd13179a90c9e0d90e15f127362c6d70161689e49fb79a1ac12b33efd128493870daa3b6918c68d0838501f73fbfbd1bccfc7faa01c5d29')
;

-- 3) Add the new betting information
INSERT INTO BETTING (bettor_id, event_id, team_id, team_rank_guess, multiplier, initmoney, betstatus) VALUES
  ('Ratishda', 'E005', 'J004', 1, 3.22, 425, 'accepted'),
  ('Tiakakuna', 'E005', 'J010', 1, 7.88, 50, 'queued for approval'),
  ('Tiakakuna', 'E005', 'J003', 3, 9.5, 230, 'queued for approval')
;

-- NOTE! Some of the given input values would automatically be generated
-- in a program logic
