/*
  MySQL SQL query script for Tournament database
  Designed for MariaDB RDBMS

  Author:
    Pekka Helenius <fincer89 [at] hotmail [dot] com>
    2019
*/

-- -----------------------------------------------------
-- Assumes that database 'tournament_db' exists
-- USE tournament_db;

-- -----------------------------------------------------
-- DATABASE USER ROLES
--
-- NOTE: This query does not use query types of [CREATE, DROP] ROLE.

-- NOTE: This query requires root permissions in the database server unless
-- current non-root user has the following permissions:
-- GRANT CREATE USER ON *.* TO '<non-root_user>'@'localhost' WITH GRANT OPTION

--
-- Create new SQL server users. Server uses local DNS address 'localhost'
--
-- IDENTIFIED BY field defines user password as plain text

-- Administration users
CREATE USER IF NOT EXISTS 'superadmin'@'localhost' IDENTIFIED BY 'supersecretpasswd';
CREATE USER IF NOT EXISTS 'loweradmin'@'localhost' IDENTIFIED BY 'lesssecretpasswd';

-- Normal users
-- Player, employee and bettor must re-define their passwords next time they log in.
CREATE USER IF NOT EXISTS 'player'@'localhost'    IDENTIFIED BY 'playerpasswd'    PASSWORD EXPIRE;
CREATE USER IF NOT EXISTS 'employee'@'localhost' IDENTIFIED BY 'employeepasswd' PASSWORD EXPIRE;
CREATE USER IF NOT EXISTS 'bettor'@'localhost' IDENTIFIED BY 'bettorpasswd' PASSWORD EXPIRE;

-- -----------------------------------------------------
-- SUPER ADMIN

-- Higher administrator has all permissions granted and can administrate
-- permissions of others.
GRANT ALL PRIVILEGES ON tournament_db.* TO 'superadmin'@'localhost'
  WITH GRANT OPTION;

-- -----------------------------------------------------
-- ADMIN

-- Lower administrator has access to all tables except for COUNTRY and CITY
-- from which he/she can't delete any rows.

-- Can't administate permissions of others.
-- GRANT ALL PRIVILEGES ON tournament_db.* TO 'loweradmin'@'localhost';

GRANT SELECT, INSERT, UPDATE ON tournament_db.COUNTRY    TO 'loweradmin'@'localhost';
GRANT SELECT, INSERT, UPDATE ON tournament_db.CITY       TO 'loweradmin'@'localhost';

GRANT ALL PRIVILEGES ON tournament_db.TEAM               TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.PLAYER             TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.ORGANIZER_ORG      TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.ORGANIZER_EMPLOYEE TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.CLUBROOM           TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.EVENT              TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.RESERVATION        TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.ARRANGEMENT        TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.TEAM_STATUS        TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.PLAYER_RANK        TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.TEAM_RANK_GLOBAL   TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.BETTOR             TO 'loweradmin'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.BETTING            TO 'loweradmin'@'localhost';

-- REVOKE DELETE        ON tournament_db.COUNTRY    FROM 'loweradmin'@'localhost';
-- REVOKE DELETE        ON CITY                     FROM 'loweradmin'@'localhost';

-- -----------------------------------------------------
-- PLAYER (in a TEAM)

-- COUNTRY: limited, undirect data representation via API
-- CITY:    limited, undirect data representation via API

-- Select and update (only) for own TEAM information
-- More fine grained role classification in a TEAM could be a player
-- with permissions to remove TEAM-related information.
GRANT SELECT, UPDATE ON tournament_db.TEAM             TO 'player'@'localhost';

-- All permissions (only) to own information
--
-- NOTE! Unimplemented feature: limit access to "own information only"
GRANT ALL PRIVILEGES ON tournament_db.PLAYER           TO 'player'@'localhost';

-- PLAYER can observe the following table data
GRANT SELECT         ON tournament_db.ORGANIZER_ORG    TO 'player'@'localhost';
GRANT SELECT         ON tournament_db.EVENT            TO 'player'@'localhost';
GRANT SELECT         ON tournament_db.TEAM_STATUS      TO 'player'@'localhost';
GRANT SELECT         ON tournament_db.TEAM_RANK_GLOBAL TO 'player'@'localhost';

-- PLAYER can observe only his/her own ranking information

-- NOTE! Unimplemented feature: limit access to "own information only"
GRANT SELECT         ON tournament_db.PLAYER_RANK      TO 'player'@'localhost';


-- PLAYER shall not have any permissions to the following tables.
--
-- Although these permissions have not been granted in the first place,
-- the REVOKE commands are demonstrated here.
--
-- REVOKE ALL           ON tournament_db.ORGANIZER_EMPLOYEE FROM 'player'@'localhost';
-- REVOKE ALL           ON tournament_db.CLUBROOM           FROM 'player'@'localhost';
-- REVOKE ALL           ON tournament_db.RESERVATION        FROM 'player'@'localhost';
-- REVOKE ALL           ON tournament_db.ARRANGEMENT        FROM 'player'@'localhost';
-- REVOKE ALL           ON tournament_db.BETTOR             FROM 'player'@'localhost';
-- REVOKE ALL           ON tournament_db.BETTING            FROM 'player'@'localhost';

-- -----------------------------------------------------
-- EMPLOYEE (in an ORGANIZATION)

-- COUNTRY: limited, undirect data representation via API
-- CITY:    limited, undirect data representation via API
-- TEAM:    limited, undirect data representation via API
-- PLAYER:  limited, undirect data representation via API

-- Select and update (only) for own ORGANIZATION information
-- More fine grained role classification in an ORGANIZATION could be an employee
-- with permissions to remove ORGANIZATION-related information.
GRANT SELECT, UPDATE ON tournament_db.ORGANIZER_ORG      TO 'employee'@'localhost';

-- All permissions (only) to own information
GRANT ALL PRIVILEGES ON tournament_db.ORGANIZER_EMPLOYEE TO 'employee'@'localhost';

-- Employee of an organization can select and update club room and game event information.
GRANT SELECT, INSERT, UPDATE ON tournament_db.CLUBROOM   TO 'employee'@'localhost';
GRANT SELECT, INSERT, UPDATE ON tournament_db.EVENT      TO 'employee'@'localhost';

-- Employee of an organization has access to all event arrangement information
-- regarding to his/her own organization.
GRANT ALL PRIVILEGES ON tournament_db.RESERVATION        TO 'employee'@'localhost';
GRANT ALL PRIVILEGES ON tournament_db.ARRANGEMENT        TO 'employee'@'localhost';

-- Employee of an organization can observe global ranking information of teams.
GRANT SELECT ON tournament_db.TEAM_RANK_GLOBAL          TO 'employee'@'localhost';

-- Employee of an organization can select and update betting information regarding
-- to bettor status information in a case the bettor is participating to
-- an event arranged by the organization.

-- NOTE! Unimplemented feature: limit access to "own information only"
GRANT SELECT, UPDATE (voittostatus) ON tournament_db.BETTING TO 'employee'@'localhost';

-- EMPLOYEE shall not have any permissions to the following tables.
--
-- Although these permissions have not been granted in the first place,
-- the REVOKE commands are demonstrated here.
--
-- REVOKE ALL           ON tournament_db.TEAM_STATUS FROM 'employee'@'localhost';
-- REVOKE ALL           ON tournament_db.PLAYER_RANK FROM 'employee'@'localhost';
-- REVOKE ALL           ON tournament_db.BETTOR      FROM 'employee'@'localhost';

-- -----------------------------------------------------
-- BETTOR

-- COUNTRY: limited, undirect data representation via API
-- CITY:    limited, undirect data representation via API
-- TEAM:    limited, undirect data representation via API

-- Select (only) for related CLUBROOM information.
-- Bettor can observe only information regarding to his/her team in a game event
-- he/she is participating.

-- NOTE! Unimplemented feature: limit access to "own information only"
GRANT SELECT         ON tournament_db.CLUBROOM         TO 'bettor'@'localhost';
GRANT SELECT         ON tournament_db.TEAM_STATUS      TO 'bettor'@'localhost';

-- Bettor can check all available and upcoming game events
-- for betting purposes.
GRANT SELECT         ON tournament_db.EVENT            TO 'bettor'@'localhost';

-- Bettor can observe global ranking of a team which helps him/her to place
-- a bet for his/her favorite team in an event.
GRANT SELECT         ON tournament_db.TEAM_RANK_GLOBAL TO 'bettor'@'localhost';

-- Bettor can fully administrate his/her profile information.

-- NOTE! Unimplemented feature: limit access to "own information only"
GRANT ALL PRIVILEGES ON tournament_db.BETTOR           TO 'bettor'@'localhost';

-- NOTE! Unimplemented feature: limit access to "own information only"

-- NOTE! Additional program logic is required regarding to events
-- and teams. This is because we need to check which possible event
-- and team values a bettor can insert into the database/input fields.

-- NOTE! Unimplemented feature: limit access to "own information only"

-- Bettor has not permission to modify his/her unique identifier or
-- BETTING win/lose status information.

GRANT SELECT, INSERT, UPDATE (event_id, team_id, team_rank_guess, multiplier, initmoney)
  ON tournament_db.BETTING TO 'bettor'@'localhost';

-- BETTOR shall not have any permissions to the following tables.
--
-- Although these permissions have not been granted in the first place,
-- the REVOKE commands are demonstrated here.
--
-- REVOKE ALL           ON tournament_db.PLAYER             FROM 'bettor'@'localhost';
-- REVOKE ALL           ON tournament_db.ORGANIZER_ORG      FROM 'bettor'@'localhost';
-- REVOKE ALL           ON tournament_db.ORGANIZER_EMPLOYEE FROM 'bettor'@'localhost';
-- REVOKE ALL           ON tournament_db.RESERVATION        FROM 'bettor'@'localhost';
-- REVOKE ALL           ON tournament_db.ARRANGEMENT        FROM 'bettor'@'localhost';
-- REVOKE ALL           ON tournament_db.PLAYER_RANK        FROM 'bettor'@'localhost';

-- -----------------------------------------------------
-- Update database server privilege definitions
-- FLUSH PRIVILEGES;

-- -----------------------------------------------------
-- Validate permission configuration
--
-- NOTE NOTE NOTE!
-- The following SELECT query is not possible for a normal database user
-- and a database server throws the following error message instead:
-- Error Code: 1044. Access denied for user <user> to database 'mysql'
--
-- Database server admin (with root permissions) should grant
-- exceptional and temporary permissions for a trusted user to 
-- run the following queries. When granting special permissions,
-- extra security precautions and protections should be considered
-- for the server.

SHOW GRANTS FOR 'superadmin'@'localhost';
SHOW GRANTS FOR 'loweradmin'@'localhost';
-- 
SHOW GRANTS FOR 'player'@'localhost';
SHOW GRANTS FOR 'employee'@'localhost';
SHOW GRANTS FOR 'bettor'@'localhost';

-- -----------------------------------------------------
-- DELETE USER ROLES
--
-- Delete created users from the database server.
-- These queries also delete user-specific, table-related
-- privilege information.

DROP USER 'superadmin'@'localhost';
DROP USER 'loweradmin'@'localhost';

DROP USER 'player'@'localhost';
DROP USER 'employee'@'localhost';
DROP USER 'bettor'@'localhost';
